@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """Top-level package for TensorFlow PhaseSpace."""
 from pkg_resources import get_distribution
 
@@ -14,6 +12,9 @@ __all__ = ["nbody_decay", "GenParticle", "random"]
 
 import tensorflow as tf
 
+from . import random
+from .phasespace import GenParticle, nbody_decay
+
 
 def _set_eager_mode():
     import os
@@ -23,6 +24,3 @@ def _set_eager_mode():
 
 
 _set_eager_mode()
-
-from . import random
-from .phasespace import nbody_decay, GenParticle
