@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 # =============================================================================
 # @file   phasespace.py
 # @author Albert Puig (albert.puig@cern.ch)
@@ -7,24 +6,24 @@
 # =============================================================================
 """Implementation of the Raubold and Lynch method to generate n-body events.
 
-The code is based on the GENBOD function (W515 from CERNLIB), documented in
-    F. James, Monte Carlo Phase Space, CERN 68-15 (1968)
+The code is based on the GENBOD function (W515 from CERNLIB), documented in:
 
+      F. James, Monte Carlo Phase Space, CERN 68-15 (1968)
 """
 import inspect
 import warnings
+from math import pi
+from typing import Callable, Dict, Optional, Tuple, Union
 
+import tensorflow as tf
+import tensorflow.experimental.numpy as tnp
+
+from . import kinematics as kin
+from .backend import function, function_jit_fixedshape
 from .random import SeedLike, get_rng
 
 RELAX_SHAPES = False
 
-from math import pi
-from typing import Union, Dict, Tuple, Optional, Callable
-
-import tensorflow as tf
-
-from . import kinematics as kin
-
 
 def process_list_to_tensor(lst):
     """Convert a list to a tensor.
@@ -39,14 +38,13 @@ def process_list_to_tensor(lst):
 
     Return:
         ~`tf.Tensor`
-
     """
     if isinstance(lst, list):
-        lst = tf.transpose(a=tf.convert_to_tensor(value=lst, dtype_hint=tf.float64))
-    return tf.cast(lst, dtype=tf.float64)
+        lst = tnp.transpose(tnp.asarray(lst, dtype=tnp.float64))
+    return tnp.asarray(lst, dtype=tnp.float64)
 
 
-@tf.function(autograph=False, experimental_relax_shapes=RELAX_SHAPES)
+@function
 def pdk(a, b, c):
     """Calculate the PDK (2-body phase space) function.
 
@@ -59,10 +57,9 @@ def pdk(a, b, c):
 
     Return:
         ~`tf.Tensor`
-
     """
     x = (a - b - c) * (a + b + c) * (a - b + c) * (a + b - c)
-    return tf.sqrt(x) / (tf.constant(2.0, dtype=tf.float64) * a)
+    return tnp.sqrt(x) / (tnp.asarray(2.0, dtype=tnp.float64) * a)
 
 
 class GenParticle:
@@ -89,7 +86,6 @@ class GenParticle:
         name (str): Name of the particle.
         mass (float, ~`tf.Tensor`, callable): Mass of the particle. If it's a float, it get
             converted to a `tf.constant`.
-
     """
 
     def __init__(self, name: str, mass: Union[Callable, int, float]) -> None:  # noqa
@@ -97,8 +93,7 @@ class GenParticle:
         self.children = []
         self._mass_val = mass
         if not callable(mass) and not isinstance(mass, tf.Variable):
-            mass = tf.convert_to_tensor(value=mass, dtype_hint=tf.float64)
-            mass = tf.cast(mass, tf.float64)
+            mass = tnp.asarray(mass, dtype=tnp.float64)
         else:
             mass = tf.function(
                 mass, autograph=False, experimental_relax_shapes=RELAX_SHAPES
@@ -129,13 +124,13 @@ class GenParticle:
             return dup_names
         return None
 
-    @tf.function(autograph=False, experimental_relax_shapes=RELAX_SHAPES)
+    @function
     def get_mass(
-            self,
-            min_mass: tf.Tensor = None,
-            max_mass: tf.Tensor = None,
-            n_events: Union[tf.Tensor, tf.Variable] = None,
-            seed: SeedLike = None,
+        self,
+        min_mass: tf.Tensor = None,
+        max_mass: tf.Tensor = None,
+        n_events: Union[tf.Tensor, tf.Variable] = None,
+        seed: SeedLike = None,
     ) -> tf.Tensor:
         """Get the particle mass.
 
@@ -157,14 +152,13 @@ class GenParticle:
 
         Raise:
             ValueError: If the mass is requested and has not been set.
-
         """
         if self.has_fixed_mass:
             mass = self._mass
         else:
             seed = get_rng(seed)
-            min_mass = tf.reshape(min_mass, (n_events,))
-            max_mass = tf.reshape(max_mass, (n_events,))
+            min_mass = tnp.reshape(min_mass, (n_events,))
+            max_mass = tnp.reshape(max_mass, (n_events,))
             signature = inspect.signature(self._mass)
             if "seed" in signature.parameters:
                 mass = self._mass(min_mass, max_mass, n_events, seed=seed)
@@ -191,7 +185,6 @@ class GenParticle:
             if children were already set, if their parent was or if less than two children were given.
             KeyError: If there is a particle name clash.
             RuntimeError: If `generate` was already called before.
-
         """
         # self._set_cache_validity(False)
         if self._generate_called:
@@ -207,7 +200,7 @@ class GenParticle:
         # Check name clashes
         name_clash = self._do_names_clash(children)
         if name_clash:
-            raise KeyError("Particle name {} already used".format(name_clash))
+            raise KeyError(f"Particle name {name_clash} already used")
         self.children = children
         return self
 
@@ -224,7 +217,6 @@ class GenParticle:
         return any(child.has_children for child in self.children)
 
     @staticmethod
-    # @tf.function(autograph=False)
     def _preprocess(momentum, n_events):
         """Preprocess momentum input and determine number of events to generate.
 
@@ -242,26 +234,21 @@ class GenParticle:
         Raise:
             tf.errors.InvalidArgumentError: If the number of events deduced from the
             shape of `momentum` is inconsistent with `n_events`.
-
         """
         momentum = process_list_to_tensor(momentum)
 
         # Check sanity of inputs
-        if momentum.shape.ndims not in (1, 2):
-            raise ValueError(
-                "Bad shape for momentum -> {}".format(momentum.shape.as_list())
-            )
+        if len(momentum.shape) not in (1, 2):
+            raise ValueError(f"Bad shape for momentum -> {list(momentum.shape)}")
         # Check compatibility of inputs
-        if momentum.shape.ndims == 2:
+        if len(momentum.shape) == 2:
             if n_events is not None:
                 momentum_shape = momentum.shape[0]
                 if momentum_shape is None:
-                    momentum_shape = tf.shape(input=momentum, out_type=tf.int64)[0]
+                    momentum_shape = tf.shape(momentum)[0]
+                    momentum_shape = tnp.asarray(momentum_shape, tnp.int64)
                 else:
-                    momentum_shape = tf.convert_to_tensor(
-                        value=momentum_shape, dtype_hint=tf.int64
-                    )
-                    momentum_shape = tf.cast(momentum_shape, dtype=tf.int64)
+                    momentum_shape = tnp.asarray(momentum_shape, dtype=tnp.int64)
                 tf.assert_equal(
                     n_events,
                     momentum_shape,
@@ -269,29 +256,29 @@ class GenParticle:
                 )
 
         if n_events is None:
-            if momentum.shape.ndims == 2:
+            if len(momentum.shape) == 2:
                 n_events = momentum.shape[0]
                 if n_events is None:  # dynamic shape
-                    n_events = tf.shape(input=momentum, out_type=tf.int64)[0]
+                    n_events = tf.shape(momentum)[0]
+                    n_events = tnp.asarray(n_events, dtype=tnp.int64)
             else:
-                n_events = tf.constant(1, dtype=tf.int64)
-        n_events = tf.convert_to_tensor(value=n_events, dtype_hint=tf.int64)
-        n_events = tf.cast(n_events, dtype=tf.int64)
+                n_events = tnp.asarray(1, dtype=tnp.int64)
+        n_events = tnp.asarray(n_events, dtype=tnp.int64)
         # Now preparation of tensors
-        if momentum.shape.ndims == 1:
-            momentum = tf.expand_dims(momentum, axis=0)
+        if len(momentum.shape) == 1:
+            momentum = tnp.expand_dims(momentum, axis=0)
         return momentum, n_events
 
     @staticmethod
-    @tf.function(autograph=False, experimental_relax_shapes=RELAX_SHAPES)
+    @function_jit_fixedshape
     def _get_w_max(available_mass, masses):
-        emmax = available_mass + tf.gather(masses, indices=[0], axis=1)
-        emmin = tf.zeros_like(emmax, dtype=tf.float64)
-        w_max = tf.ones_like(emmax, dtype=tf.float64)
+        emmax = available_mass + tnp.take(masses, indices=[0], axis=1)
+        emmin = tnp.zeros_like(emmax, dtype=tnp.float64)
+        w_max = tnp.ones_like(emmax, dtype=tnp.float64)
         for i in range(1, masses.shape[1]):
-            emmin += tf.gather(masses, [i - 1], axis=1)
-            emmax += tf.gather(masses, [i], axis=1)
-            w_max *= pdk(emmax, emmin, tf.gather(masses, [i], axis=1))
+            emmin += tnp.take(masses, [i - 1], axis=1)
+            emmax += tnp.take(masses, [i], axis=1)
+            w_max *= pdk(emmax, emmin, tnp.take(masses, [i], axis=1))
         return w_max
 
     def _generate(self, momentum, n_events, rng):
@@ -310,13 +297,12 @@ class GenParticle:
         Return:
             tuple: Result of the generation (per-event weights, maximum weights, output particles
                 and their output masses).
-
         """
         self._generate_called = True
         if not self.children:
             raise ValueError("No children have been configured")
         p_top, n_events = self._preprocess(momentum, n_events)
-        top_mass = tf.broadcast_to(kin.mass(p_top), (n_events, 1))
+        top_mass = tnp.broadcast_to(kin.mass(p_top), (n_events, 1))
         n_particles = len(self.children)
 
         # Prepare masses
@@ -329,11 +315,9 @@ class GenParticle:
                     output_mass += recurse_stable(child)
             return output_mass
 
-        mass_from_stable = tf.broadcast_to(
-            tf.reduce_sum(
-                input_tensor=[
-                    child.get_mass() for child in self.children if child.has_fixed_mass
-                ],
+        mass_from_stable = tnp.broadcast_to(
+            tnp.sum(
+                [child.get_mass() for child in self.children if child.has_fixed_mass],
                 axis=0,
             ),
             (n_events, 1),
@@ -342,47 +326,45 @@ class GenParticle:
         masses = []
         for child in self.children:
             if child.has_fixed_mass:
-                masses.append(tf.broadcast_to(child.get_mass(), (n_events, 1)))
+                masses.append(tnp.broadcast_to(child.get_mass(), (n_events, 1)))
             else:
                 # Recurse that particle to know the minimum mass we need to generate
-                min_mass = tf.broadcast_to(recurse_stable(child), (n_events, 1))
+                min_mass = tnp.broadcast_to(recurse_stable(child), (n_events, 1))
                 mass = child.get_mass(min_mass, max_mass, n_events)
-                mass = tf.reshape(mass, (n_events, 1))
+                mass = tnp.reshape(mass, (n_events, 1))
                 max_mass -= mass
                 masses.append(mass)
-        masses = tf.concat(masses, axis=-1)
-        # if masses.shape.ndims == 1:
-        #     masses = tf.expand_dims(masses, axis=0)
-        available_mass = top_mass - tf.reduce_sum(
-            input_tensor=masses, axis=1, keepdims=True
-        )
+        masses = tnp.concatenate(masses, axis=-1)
+        # if len(masses.shape) == 1:
+        #     masses = tnp.expand_dims(masses, axis=0)
+        available_mass = top_mass - tnp.sum(masses, axis=1, keepdims=True)
         tf.debugging.assert_greater_equal(
             available_mass,
-            tf.zeros_like(available_mass, dtype=tf.float64),
+            tnp.zeros_like(available_mass, dtype=tnp.float64),
             message="Forbidden decay",
             name="mass_check",
         )  # Calculate the max weight, initial beta, etc
         w_max = self._get_w_max(available_mass, masses)
         p_top_boost = kin.boost_components(p_top)
         # Start the generation
-        random_numbers = rng.uniform((n_events, n_particles - 2), dtype=tf.float64)
-        random = tf.concat(
+        random_numbers = rng.uniform((n_events, n_particles - 2), dtype=tnp.float64)
+        random = tnp.concatenate(
             [
-                tf.zeros((n_events, 1), dtype=tf.float64),
-                tf.sort(random_numbers, axis=1),
-                tf.ones((n_events, 1), dtype=tf.float64),
+                tnp.zeros((n_events, 1), dtype=tnp.float64),
+                tnp.sort(random_numbers, axis=1),
+                tnp.ones((n_events, 1), dtype=tnp.float64),
             ],
             axis=1,
         )
         if random.shape[1] is None:
             random.set_shape((None, n_particles))
-        # random = tf.expand_dims(random, axis=-1)
-        sum_ = tf.zeros((n_events, 1), dtype=tf.float64)
+        # random = tnp.expand_dims(random, axis=-1)
+        sum_ = tnp.zeros((n_events, 1), dtype=tnp.float64)
         inv_masses = []
         # TODO(Mayou36): rewrite with cumsum?
         for i in range(n_particles):
-            sum_ += tf.gather(masses, [i], axis=1)
-            inv_masses.append(tf.gather(random, [i], axis=1) * available_mass + sum_)
+            sum_ += tnp.take(masses, [i], axis=1)
+            inv_masses.append(tnp.take(random, [i], axis=1) * available_mass + sum_)
         generated_particles, weights = self._generate_part2(
             inv_masses, masses, n_events, n_particles, rng=rng
         )
@@ -391,33 +373,35 @@ class GenParticle:
             kin.lorentz_boost(part, p_top_boost) for part in generated_particles
         ]
         return (
-            tf.reshape(weights, (n_events,)),
-            tf.reshape(w_max, (n_events,)),
+            tnp.reshape(weights, (n_events,)),
+            tnp.reshape(w_max, (n_events,)),
             generated_particles,
             masses,
         )
 
     @staticmethod
-    @tf.function(autograph=False, experimental_relax_shapes=RELAX_SHAPES)
+    @function
     def _generate_part2(inv_masses, masses, n_events, n_particles, rng):
         pds = []
         # Calculate weights of the events
         for i in range(n_particles - 1):
             pds.append(
                 pdk(
-                    inv_masses[i + 1], inv_masses[i], tf.gather(masses, [i + 1], axis=1)
+                    inv_masses[i + 1],
+                    inv_masses[i],
+                    tnp.take(masses, [i + 1], axis=1),
                 )
             )
-        weights = tf.reduce_prod(input_tensor=pds, axis=0)
-        zero_component = tf.zeros_like(pds[0], dtype=tf.float64)
+        weights = tnp.prod(pds, axis=0)
+        zero_component = tnp.zeros_like(pds[0], dtype=tnp.float64)
         generated_particles = [
-            tf.concat(
+            tnp.concatenate(
                 [
                     zero_component,
                     pds[0],
                     zero_component,
-                    tf.sqrt(
-                        tf.square(pds[0]) + tf.square(tf.gather(masses, [0], axis=1))
+                    tnp.sqrt(
+                        tnp.square(pds[0]) + tnp.square(tnp.take(masses, [0], axis=1))
                     ),
                 ],
                 axis=1,
@@ -426,38 +410,38 @@ class GenParticle:
         part_num = 1
         while True:
             generated_particles.append(
-                tf.concat(
+                tnp.concatenate(
                     [
                         zero_component,
                         -pds[part_num - 1],
                         zero_component,
-                        tf.sqrt(
-                            tf.square(pds[part_num - 1])
-                            + tf.square(tf.gather(masses, [part_num], axis=1))
+                        tnp.sqrt(
+                            tnp.square(pds[part_num - 1])
+                            + tnp.square(tnp.take(masses, [part_num], axis=1))
                         ),
                     ],
                     axis=1,
                 )
             )
-            # with tf.control_dependencies([n_events]):
-            cos_z = tf.constant(2.0, dtype=tf.float64) * rng.uniform(
-                (n_events, 1), dtype=tf.float64
-            ) - tf.constant(1.0, dtype=tf.float64)
-            sin_z = tf.sqrt(tf.constant(1.0, dtype=tf.float64) - cos_z * cos_z)
+
+            cos_z = tnp.asarray(2.0, dtype=tnp.float64) * rng.uniform(
+                (n_events, 1), dtype=tnp.float64
+            ) - tnp.asarray(1.0, dtype=tnp.float64)
+            sin_z = tnp.sqrt(tnp.asarray(1.0, dtype=tnp.float64) - cos_z * cos_z)
             ang_y = (
-                    tf.constant(2.0, dtype=tf.float64)
-                    * tf.constant(pi, dtype=tf.float64)
-                    * rng.uniform((n_events, 1), dtype=tf.float64)
+                tnp.asarray(2.0, dtype=tnp.float64)
+                * tnp.asarray(pi, dtype=tnp.float64)
+                * rng.uniform((n_events, 1), dtype=tnp.float64)
             )
-            cos_y = tf.math.cos(ang_y)
-            sin_y = tf.math.sin(ang_y)
+            cos_y = tnp.cos(ang_y)
+            sin_y = tnp.sin(ang_y)
             # Do the rotations
             for j in range(part_num + 1):
                 px = kin.x_component(generated_particles[j])
                 py = kin.y_component(generated_particles[j])
                 # Rotate about z
                 # TODO(Mayou36): only list? will be overwritten below anyway, but can `*_component` handle it?
-                generated_particles[j] = tf.concat(
+                generated_particles[j] = tnp.concatenate(
                     [
                         cos_z * px - sin_z * py,
                         sin_z * px + cos_z * py,
@@ -469,7 +453,7 @@ class GenParticle:
                 # Rotate about y
                 px = kin.x_component(generated_particles[j])
                 pz = kin.z_component(generated_particles[j])
-                generated_particles[j] = tf.concat(
+                generated_particles[j] = tnp.concatenate(
                     [
                         cos_y * px - sin_y * pz,
                         kin.y_component(generated_particles[j]),
@@ -480,25 +464,26 @@ class GenParticle:
                 )
             if part_num == (n_particles - 1):
                 break
-            betas = pds[part_num] / tf.sqrt(
-                tf.square(pds[part_num]) + tf.square(inv_masses[part_num])
+            betas = pds[part_num] / tnp.sqrt(
+                tnp.square(pds[part_num]) + tnp.square(inv_masses[part_num])
             )
             generated_particles = [
                 kin.lorentz_boost(
-                    part, tf.concat([zero_component, betas, zero_component], axis=1)
+                    part,
+                    tnp.concatenate([zero_component, betas, zero_component], axis=1),
                 )
                 for part in generated_particles
             ]
             part_num += 1
         return generated_particles, weights
 
-    @tf.function(autograph=False, experimental_relax_shapes=True)
+    @function
     def _recursive_generate(
-            self,
-            n_events,
-            boost_to=None,
-            recalculate_max_weights=False,
-            rng: SeedLike = None,
+        self,
+        n_events,
+        boost_to=None,
+        recalculate_max_weights=False,
+        rng: SeedLike = None,
     ):
         """Recursively generate normalized n-body phase space as tensorflow tensors.
 
@@ -526,15 +511,15 @@ class GenParticle:
 
         Raise:
             tf.errors.InvalidArgumentError: If the the decay is kinematically forbidden.
-            ValueError: If `n_events` and the size of `boost_to` don't match. See `GenParticle.generate_unnormalized`.
-
+            ValueError: If `n_events` and the size of `boost_to` don't match.
+            See `GenParticle.generate_unnormalized`.
         """
         if boost_to is not None:
             momentum = boost_to
         else:
             if self.has_fixed_mass:
-                momentum = tf.broadcast_to(
-                    tf.stack((0.0, 0.0, 0.0, self.get_mass()), axis=-1), (n_events, 4)
+                momentum = tnp.broadcast_to(
+                    tnp.stack((0.0, 0.0, 0.0, self.get_mass()), axis=-1), (n_events, 4)
                 )
             else:
                 raise ValueError("Cannot use resonance as top particle")
@@ -546,7 +531,7 @@ class GenParticle:
             for child_num, child in enumerate(self.children)
         }
         output_masses = {
-            child.name: tf.gather(children_masses, [child_num], axis=1)
+            child.name: tnp.take(children_masses, [child_num], axis=1)
             for child_num, child in enumerate(self.children)
         }
         for child_num, child in enumerate(self.children):
@@ -589,7 +574,7 @@ class GenParticle:
                     get_flattened_values(current_mass_tree)
                 )
                 masses = []
-                w_max = tf.ones_like(available_mass)
+                w_max = tnp.ones_like(available_mass)
                 for child, child_mass in current_mass_tree.items():
                     if isinstance(child_mass, dict):
                         w_max *= recurse_w_max(
@@ -608,27 +593,26 @@ class GenParticle:
                         masses.append(sum(get_flattened_values(child_mass)))
                     else:
                         masses.append(child_mass)
-                masses = tf.concat(masses, axis=1)
+                masses = tnp.concatenate(masses, axis=1)
                 w_max *= self._get_w_max(available_mass, masses)
                 return w_max
 
             mass_tree = {}
             build_mass_tree(self, mass_tree)
             momentum = process_list_to_tensor(momentum)
-            if len(momentum.shape.as_list()) == 1:
-                momentum = tf.expand_dims(momentum, axis=-1)
-            weights_max = tf.reshape(
+            if len(momentum.shape) == 1:
+                momentum = tnp.expand_dims(momentum, axis=-1)
+            weights_max = tnp.reshape(
                 recurse_w_max(kin.mass(momentum), mass_tree[self.name]), (n_events,)
             )
         return weights, weights_max, output_particles, output_masses
 
-    # @tf.function(autograph=False)
     def generate(
-            self,
-            n_events: Union[int, tf.Tensor, tf.Variable],
-            boost_to: Optional[tf.Tensor] = None,
-            normalize_weights: bool = True,
-            seed: SeedLike = None,
+        self,
+        n_events: Union[int, tf.Tensor, tf.Variable],
+        boost_to: Optional[tf.Tensor] = None,
+        normalize_weights: bool = True,
+        seed: SeedLike = None,
     ) -> Tuple[tf.Tensor, Dict[str, tf.Tensor]]:
         """Generate normalized n-body phase space as tensorflow tensors.
 
@@ -653,18 +637,18 @@ class GenParticle:
             tuple: Result of the generation, which varies with the value of `normalize_weights`:
 
                 + If True, the tuple elements are the normalized event weights as a tensor of shape
-                (n_events, ), and the momenta generated particles as a dictionary of tensors of shape
-                (4, n_events) with particle names as keys.
+                  (n_events, ), and the momenta generated particles as a dictionary of tensors of shape
+                  (4, n_events) with particle names as keys.
 
                 + If False, the tuple weights are the unnormalized event weights as a tensor of shape
-                (n_events, ), the maximum per-event weights as a tensor of shape (n_events, ) and the
-                momenta generated particles as a dictionary of tensors of shape (4, n_events) with particle
-                names as keys.
+                  (n_events, ), the maximum per-event weights as a tensor of shape (n_events, ) and the
+                  momenta generated particles as a dictionary of tensors of shape (4, n_events) with particle
+                  names as keys.
 
         Raise:
             tf.errors.InvalidArgumentError: If the the decay is kinematically forbidden.
-            ValueError: If `n_events` and the size of `boost_to` don't match. See `GenParticle.generate_unnormalized`.
-
+            ValueError: If `n_events` and the size of `boost_to` don't match.
+            See `GenParticle.generate_unnormalized`.
         """
         rng = get_rng(seed)
         if boost_to is not None:
@@ -672,12 +656,9 @@ class GenParticle:
                 f"The number of events requested ({n_events}) doesn't match the boost_to input size "
                 f"of {boost_to.shape}"
             )
-            tf.assert_equal(
-                tf.shape(input=boost_to)[0], tf.shape(input=n_events), message=message
-            )
+            tf.assert_equal(tf.shape(boost_to)[0], tf.shape(n_events), message=message)
         if not isinstance(n_events, tf.Variable):
-            n_events = tf.convert_to_tensor(value=n_events, dtype_hint=tf.int64)
-            n_events = tf.cast(n_events, dtype=tf.int64)
+            n_events = tnp.asarray(n_events, dtype=tnp.int64)
         weights, weights_max, parts, _ = self._recursive_generate(
             n_events=n_events,
             boost_to=boost_to,
@@ -691,10 +672,10 @@ class GenParticle:
         )
 
     def generate_tensor(
-            self,
-            n_events: int,
-            boost_to=None,
-            normalize_weights: bool = True,
+        self,
+        n_events: int,
+        boost_to=None,
+        normalize_weights: bool = True,
     ):
         """Generate normalized n-body phase space as numpy arrays.
 
@@ -715,18 +696,18 @@ class GenParticle:
             tuple: Result of the generation, which varies with the value of `normalize_weights`:
 
                 + If True, the tuple elements are the normalized event weights as an array of shape
-                (n_events, ), and the momenta generated particles as a dictionary of arrays of shape
-                (4, n_events) with particle names as keys.
+                  (n_events, ), and the momenta generated particles as a dictionary of arrays of shape
+                  (4, n_events) with particle names as keys.
 
                 + If False, the tuple weights are the unnormalized event weights as an array of shape
-                (n_events, ), the maximum per-event weights as an array of shape (n_events, ) and the
-                momenta generated particles as a dictionary of arrays of shape (4, n_events) with particle
-                names as keys.
+                  (n_events, ), the maximum per-event weights as an array of shape (n_events, ) and the
+                  momenta generated particles as a dictionary of arrays of shape (4, n_events) with particle
+                  names as keys.
 
         Raise:
             tf.errors.InvalidArgumentError: If the the decay is kinematically forbidden.
-            ValueError: If `n_events` and the size of `boost_to` don't match. See `GenParticle.generate_unnormalized`.
-
+            ValueError: If `n_events` and the size of `boost_to` don't match.
+            See `GenParticle.generate_unnormalized`.
         """
 
         # Run generation
@@ -744,7 +725,6 @@ class Particle:
     """Deprecated Particle class.
 
     Renamed to GenParticle.
-
     """
 
     def __init__(self):
@@ -773,7 +753,6 @@ def nbody_decay(mass_top: float, masses: list, top_name: str = "", names: list =
 
     Raise:
         ValueError: If the length of `masses` and `names` doesn't match.
-
     """
     if not top_name:
         top_name = "top"
@@ -793,4 +772,5 @@ def generate_decay(*args, **kwargs):
         "For more information see https://github.com/zfit/phasespace/issues/22"
     )
 
+
 # EOF
